//
//  File.swift
//  
//
//  Created by Max Nuding on 04.12.21.
//

import Foundation

struct Board {
    var numbers: [[Int?]]
    var hasWon = false
    
    mutating func crossOf(num: Int) -> Bool {
        for (rowNumber, row) in numbers.enumerated() {
            if let colNumber = row.firstIndex(of: num) {
                numbers[rowNumber][colNumber] = nil
                //print("Found at: \(rowNumber),\(colNumber)")
                return true
            }
        }
        return false
    }
    
    mutating func checkHasWon() -> Bool {
        //print("HasWon?: \(numbers)")
        let foundInRows = numbers.contains { row in
            row.allSatisfy { $0 == nil }
        }
        if foundInRows {
            return true
        }
        let foundInColumns = numbers.indices.contains { columnIndex in
            return numbers.allSatisfy { $0[columnIndex] == nil }
        }
        hasWon = foundInColumns
        return hasWon
    }
    
    func calculateScore() -> Int {
        numbers
            .reduce([], +)
            .compactMap { $0 }
            .reduce(0, +)
    }
}

struct Day04 {
    let inputPath: String
    
    func run() {
        let input = try! String(contentsOfFile: inputPath)
        var lines = input
            .components(separatedBy: .newlines)
        let inputNumbers = lines
            .removeFirst()
            .split(separator: ",")
            .map { Int($0)! }
        _ = lines.removeFirst() //empty seperator line
        
        let boards = lines
            .split(separator: "")
            .map { boardLines in
                Board(numbers: boardLines.map { $0.split(separator: " ").map { num in Int(num)! } })
            }
        var boardsA = boards
        runA(boards: &boardsA, inputNumbers: inputNumbers)
    }
    
    func runA(boards: inout [Board], inputNumbers: [Int]) {
        for inputNumber in inputNumbers {
            //print("\nInput: \(inputNumber)")
            for (idx, var board) in boards.enumerated() {
                //print("Checking board: \(idx)")
                if board.crossOf(num: inputNumber) && board.checkHasWon() {
                    print(board.calculateScore() * inputNumber)
                    return
                }
                boards[idx] = board
            }
        }
    }
    
    func runB(_ commands: [String]) {
        
    }
}
